TEACHBOOKS_WORK_DIR = ".teachbooks/"
SERVER_WORK_DIR = TEACHBOOKS_WORK_DIR + "server/"
RELEASE_WORK_DIR = TEACHBOOKS_WORK_DIR + "release/"
BOOK_DIR = "book/"
BUILD_DIR = "_build/html/"
BOOK_SERVE_DIR = BOOK_DIR + BUILD_DIR
